'use client';
import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { Fragment, lazy, useEffect, useMemo, useState, } from 'react';
import { Controller, FormProvider, useForm, useFormContext, } from 'react-hook-form';
import { useApiContext, useServerSelectContext } from '../ui/contexts/api.js';
import { FieldSet, JsonInput } from './inputs.js';
import { getStatusInfo } from './status-info.js';
import { getUrl } from '../utils/server-url.js';
import { DynamicCodeBlock } from 'fumadocs-ui/components/dynamic-codeblock';
import { MethodLabel } from '../ui/components/method-label.js';
import { useQuery } from '../utils/use-query.js';
import { Collapsible, CollapsibleContent, CollapsibleTrigger, } from 'fumadocs-ui/components/ui/collapsible';
import { ChevronDown, LoaderCircle } from '../icons.js';
import { useRequestData } from '../ui/contexts/code-example.js';
import { useEffectEvent } from 'fumadocs-core/utils/use-effect-event';
import { buttonVariants } from 'fumadocs-ui/components/ui/button';
import { cn } from 'fumadocs-ui/utils/cn';
import { SchemaProvider, useResolvedSchema, } from '../playground/schema.js';
function toRequestData(method, mediaType, value) {
    return {
        path: value.path,
        method,
        header: value.header,
        body: value.body,
        bodyMediaType: mediaType,
        cookie: value.cookie,
        query: value.query,
    };
}
const ServerSelect = lazy(() => import('../ui/server-select.js'));
export default function Client({ route, method = 'GET', authorization, parameters, body, fields, references, proxyUrl, components: { ResultDisplay = DefaultResultDisplay } = {}, ...rest }) {
    const { server } = useServerSelectContext();
    const requestData = useRequestData();
    const fieldInfoMap = useMemo(() => new Map(), []);
    const authInfo = usePersistentAuthInfo(authorization);
    const { mediaAdapters } = useApiContext();
    const defaultValues = useMemo(() => ({
        authorization: authInfo.info,
        path: requestData.data.path,
        query: requestData.data.query,
        header: requestData.data.header,
        body: requestData.data.body,
        cookie: requestData.data.cookie,
    }), [authInfo.info, requestData.data]);
    const form = useForm({
        defaultValues,
    });
    const testQuery = useQuery(async (input) => {
        const fetcher = await import('./fetcher.js').then((mod) => mod.createBrowserFetcher(mediaAdapters));
        const serverUrl = server
            ? getUrl(server.url, server.variables)
            : window.location.origin;
        return fetcher.fetch(`${serverUrl}${route}`, {
            proxyUrl,
            ...toRequestData(method, body?.mediaType, input),
        });
    });
    useEffect(() => {
        fieldInfoMap.clear();
        form.reset(defaultValues);
        // eslint-disable-next-line react-hooks/exhaustive-deps -- update default value
    }, [defaultValues]);
    useEffect(() => {
        const subscription = form.watch((_value) => {
            const value = _value;
            if (authorization && value.authorization) {
                authInfo.saveInfo(value.authorization);
                writeAuthHeader(authorization, value.authorization, value.header, value.query, value.cookie);
            }
            requestData.saveData(toRequestData(method, body?.mediaType, value));
        });
        return () => {
            subscription.unsubscribe();
        };
        // eslint-disable-next-line react-hooks/exhaustive-deps -- mounted only once
    }, []);
    const onSubmit = form.handleSubmit((value) => {
        testQuery.start(value);
    });
    return (_jsx(FormProvider, { ...form, children: _jsx(SchemaProvider, { fieldInfoMap: fieldInfoMap, references: references, children: _jsxs("form", { ...rest, className: cn('not-prose flex flex-col rounded-xl border shadow-md overflow-hidden bg-fd-card text-fd-card-foreground', rest.className), onSubmit: onSubmit, children: [_jsx(ServerSelect, {}), _jsxs("div", { className: "flex flex-row items-center gap-2 text-sm p-3 pb-0", children: [_jsx(MethodLabel, { children: method }), _jsx(Route, { route: route, className: "flex-1" }), _jsx("button", { type: "submit", className: cn(buttonVariants({ color: 'primary', size: 'sm' }), 'px-3 py-1.5'), disabled: testQuery.isLoading, children: testQuery.isLoading ? (_jsx(LoaderCircle, { className: "size-4 animate-spin" })) : ('Send') })] }), _jsx(FormBody, { body: body, fields: fields, parameters: parameters, authorization: authorization }), testQuery.data ? _jsx(ResultDisplay, { data: testQuery.data }) : null, authorization && _jsx(AuthFooter, { authorization: authorization })] }) }) }));
}
const paramNames = ['Headers', 'Cookies', 'Query', 'Path'];
const paramTypes = ['header', 'cookie', 'query', 'path'];
function FormBody({ authorization, parameters = [], fields = {}, body, }) {
    const params = useMemo(() => {
        return paramTypes.map((param) => parameters.filter((v) => v.in === param));
    }, [parameters]);
    function renderAuth() {
        if (!authorization)
            return null;
        const schema = authorization.type === 'http' && authorization.scheme === 'basic'
            ? {
                type: 'object',
                required: ['username', 'password'],
                properties: {
                    username: {
                        type: 'string',
                    },
                    password: {
                        type: 'string',
                    },
                },
            }
            : {
                type: 'string',
                description: 'The Authorization access token',
            };
        if (fields?.auth)
            return renderCustomField('authorization', schema, fields.auth);
        return (_jsx(FieldSet, { fieldName: "authorization", name: "Authorization", field: schema, isRequired: true }));
    }
    return (_jsxs(_Fragment, { children: [params.map((param, i) => {
                const name = paramNames[i];
                const type = paramTypes[i];
                if (type !== 'header' && param.length === 0)
                    return;
                if (type === 'header' && !authorization && param.length === 0)
                    return;
                return (_jsxs(CollapsiblePanel, { title: name, children: [type === 'header' ? renderAuth() : null, param.map((field) => {
                            const fieldName = `${type}.${field.name}`;
                            if (fields?.parameter) {
                                return renderCustomField(fieldName, field.schema, fields.parameter, field.name);
                            }
                            return (_jsx(FieldSet, { name: field.name, fieldName: fieldName, field: field.schema }, fieldName));
                        })] }, name));
            }), body && (_jsx(CollapsiblePanel, { title: "Body", children: fields.body ? (renderCustomField('body', body.schema, fields.body)) : (_jsx(BodyInput, { field: body.schema })) }))] }));
}
function BodyInput({ field: _field }) {
    const field = useResolvedSchema(_field);
    const [isJson, setIsJson] = useState(false);
    if (field.format === 'binary')
        return _jsx(FieldSet, { field: field, fieldName: "body" });
    return (_jsx(_Fragment, { children: isJson ? (_jsx(JsonInput, { fieldName: "body", children: _jsx("button", { className: cn(buttonVariants({
                    color: 'ghost',
                    size: 'sm',
                    className: 'p-2',
                })), onClick: () => setIsJson(false), type: "button", children: "Close JSON Editor" }) })) : (_jsx(FieldSet, { field: field, fieldName: "body", name: _jsx("button", { className: cn(buttonVariants({
                    color: 'secondary',
                    size: 'sm',
                    className: 'p-2',
                })), onClick: () => setIsJson(true), type: "button", children: "Open JSON Editor" }) })) }));
}
function renderCustomField(fieldName, info, field, key) {
    return (_jsx(Controller, { 
        // @ts-expect-error we use string here
        render: (props) => field.render({ ...props, info }), name: fieldName }, key));
}
const OauthDialog = lazy(() => import('./auth/oauth-dialog.js').then((mod) => ({
    default: mod.OauthDialog,
})));
const OauthDialogTrigger = lazy(() => import('./auth/oauth-dialog.js').then((mod) => ({
    default: mod.OauthDialogTrigger,
})));
function AuthFooter({ authorization }) {
    const form = useFormContext();
    if (authorization.type !== 'oauth2')
        return;
    // only the first one, so it looks simpler :)
    const flow = Object.keys(authorization.flows)[0];
    return (_jsx(OauthDialog, { flow: flow, scheme: authorization, setToken: (token) => form.setValue('authorization', `Bearer ${token}`), children: _jsx(OauthDialogTrigger, { type: "button", className: cn(buttonVariants({
                color: 'secondary',
            })), children: "Auth Settings" }) }));
}
function Route({ route, ...props }) {
    const segments = route.split('/').filter((part) => part.length > 0);
    return (_jsx("div", { ...props, className: cn('flex flex-row items-center gap-0.5 overflow-auto text-nowrap', props.className), children: segments.map((part, index) => (_jsxs(Fragment, { children: [_jsx("span", { className: "text-fd-muted-foreground", children: "/" }), part.startsWith('{') && part.endsWith('}') ? (_jsx("code", { className: "bg-fd-primary/10 text-fd-primary", children: part })) : (_jsx("code", { className: "text-fd-foreground", children: part }))] }, index))) }));
}
function DefaultResultDisplay({ data }) {
    const statusInfo = useMemo(() => getStatusInfo(data.status), [data.status]);
    const { shikiOptions } = useApiContext();
    return (_jsxs("div", { className: "flex flex-col gap-3 border-t p-3", children: [_jsxs("div", { className: "inline-flex items-center gap-1.5 text-sm font-medium text-fd-foreground", children: [_jsx(statusInfo.icon, { className: cn('size-4', statusInfo.color) }), statusInfo.description] }), _jsx("p", { className: "text-sm text-fd-muted-foreground", children: data.status }), data.data ? (_jsx(DynamicCodeBlock, { lang: typeof data.data === 'string' && data.data.length > 50000
                    ? 'text'
                    : data.type, code: typeof data.data === 'string'
                    ? data.data
                    : JSON.stringify(data.data, null, 2), options: shikiOptions })) : null] }));
}
function usePersistentAuthInfo(authorization) {
    const key = authorization
        ? `__fumadocs_auth_${authorization.persistentId}`
        : null;
    const [info, setInfo] = useState(() => {
        if (!authorization || authorization.type === 'apiKey')
            return '';
        if (authorization.type === 'http' && authorization.scheme === 'basic') {
            return {
                username: '',
                password: '',
            };
        }
        return 'Bearer';
    });
    useEffect(() => {
        if (!key)
            return;
        const item = localStorage.getItem(key);
        if (item) {
            setInfo(JSON.parse(item));
        }
    }, [key]);
    return {
        info,
        saveInfo: useEffectEvent((value) => {
            if (!key)
                return;
            localStorage.setItem(key, JSON.stringify(value));
        }),
    };
}
function CollapsiblePanel({ title, children, ...props }) {
    return (_jsxs(Collapsible, { ...props, className: "border-b last:border-b-0", children: [_jsxs(CollapsibleTrigger, { className: "group w-full flex items-center gap-2 p-3 text-sm font-medium", children: [title, _jsx(ChevronDown, { className: "ms-auto size-3.5 text-fd-muted-foreground group-data-[state=open]:rotate-180" })] }), _jsx(CollapsibleContent, { children: _jsx("div", { className: "flex flex-col gap-3 p-3 pt-1", children: children }) })] }));
}
function writeAuthHeader(authorization, input, header, query, cookie) {
    if (authorization.type === 'apiKey') {
        if (authorization.in === 'header') {
            header[authorization.name] = input;
        }
        if (authorization.in === 'query') {
            query[authorization.name] = input;
        }
        if (authorization.in === 'cookie') {
            cookie[authorization.name] = input;
        }
        return;
    }
    if (authorization.type === 'http' &&
        authorization.scheme === 'basic' &&
        typeof input === 'object') {
        header.Authorization = `Basic ${btoa(`${input.username}:${input.password}`)}`;
        return;
    }
    if (typeof input === 'string') {
        header.Authorization = input;
    }
}
