import { jsx as _jsx } from "react/jsx-runtime";
import { getPreferredType } from '../utils/schema.js';
import { getSecurities } from '../utils/get-security.js';
import { ClientLazy } from '../ui/lazy.js';
export async function APIPlayground({ path, method, ctx, client, }) {
    let currentId = 0;
    const bodyContent = method.requestBody?.content;
    const mediaType = bodyContent ? getPreferredType(bodyContent) : undefined;
    const context = {
        references: {},
        nextId() {
            return String(currentId++);
        },
        registered: new WeakMap(),
    };
    const props = {
        authorization: getAuthorizationField(method, ctx),
        method: method.method,
        route: path,
        parameters: method.parameters?.map((v) => ({
            name: v.name,
            in: v.in,
            schema: writeReferences((v.schema ?? true), context),
            description: v.description,
        })),
        body: bodyContent && mediaType
            ? {
                schema: writeReferences(bodyContent[mediaType].schema, context),
                mediaType,
            }
            : undefined,
        references: context.references,
        proxyUrl: ctx.proxyUrl,
        ...client,
    };
    return _jsx(ClientLazy, { ...props });
}
function writeReferences(schema, ctx, stack = new WeakMap()) {
    if (typeof schema !== 'object' || !schema)
        return schema;
    if (stack.has(schema)) {
        const out = stack.get(schema);
        const id = ctx.nextId();
        ctx.references[id] = out;
        return {
            $ref: id,
        };
    }
    const output = { ...schema };
    stack.set(schema, output);
    for (const name of ['items', 'additionalProperties']) {
        if (!output[name])
            continue;
        output[name] = writeReferences(output[name], ctx, stack);
    }
    for (const name of ['oneOf', 'allOf', 'anyOf']) {
        if (!output[name])
            continue;
        output[name] = output[name].map((item) => writeReferences(item, ctx, stack));
    }
    for (const name of ['properties', 'patternProperties']) {
        if (!output[name])
            continue;
        output[name] = { ...output[name] };
        for (const key in output[name]) {
            output[name][key] = writeReferences(output[name][key], ctx, stack);
        }
    }
    return output;
}
function getAuthorizationField(method, { schema: { document } }) {
    const security = method.security ?? document.security ?? [];
    if (security.length === 0)
        return;
    let item;
    for (const requirements of security) {
        const keys = Object.keys(requirements).length;
        if (keys === 0)
            return;
        else if (keys === 1)
            item = requirements;
    }
    if (!item) {
        console.warn(`Cannot find suitable security scheme for API Playground from ${JSON.stringify(security, null, 2)}. Only schemes with one requirement are allowed.`);
        return;
    }
    const scheme = getSecurities(item, document)[0];
    if (scheme.type === 'oauth2') {
        const flow = Object.keys(scheme.flows).at(0);
        if (!flow)
            throw new Error("security scheme's `flows` must not be empty");
        if (flow === 'implicit' || flow === 'password')
            throw new Error(`OAuth 2.0 flow type: ${flow} is not supported, consider other types like \`authorizationCode\` instead.`);
    }
    return {
        persistentId: Object.keys(item)[0],
        ...scheme,
    };
}
