import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
import { Fragment } from 'react';
import { getPreferredType, } from '../../utils/schema.js';
import { getSecurities, getSecurityPrefix } from '../../utils/get-security.js';
import { idToTitle } from '../../utils/id-to-title.js';
import { Markdown } from '../markdown.js';
import { heading } from '../heading.js';
import { Schema } from '../schema.js';
import { createMethod } from '../../server/create-method.js';
import { methodKeys } from '../../build-routes.js';
import { APIExample, APIExampleProvider, getAPIExamples, } from '../../render/operation/api-example.js';
import { MethodLabel } from '../../ui/components/method-label.js';
import { Tab, Tabs } from 'fumadocs-ui/components/tabs';
import { getTypescriptSchema } from '../../utils/get-typescript-schema.js';
import { CopyResponseTypeScript } from '../../ui/client.js';
const ParamTypes = {
    path: 'Path Parameters',
    query: 'Query Parameters',
    header: 'Header Parameters',
    cookie: 'Cookie Parameters',
};
export function Operation({ type = 'operation', path, method, ctx, hasHead, headingLevel = 2, }) {
    const body = method.requestBody;
    let headNode = null;
    let bodyNode = null;
    let responseNode = null;
    let callbacksNode = null;
    if (hasHead) {
        const title = method.summary ??
            (method.operationId ? idToTitle(method.operationId) : path);
        headNode = (_jsxs(_Fragment, { children: [heading(headingLevel, title, ctx), method.description ? _jsx(Markdown, { text: method.description }) : null] }));
        headingLevel++;
    }
    if (body) {
        const type = getPreferredType(body.content);
        if (!type || !(type in ctx.mediaAdapters))
            throw new Error(`No supported media type for body content: ${path}, received: ${type}`);
        bodyNode = (_jsxs(_Fragment, { children: [heading(headingLevel, 'Request Body', ctx), _jsxs("div", { className: "mb-4 p-3 bg-fd-card rounded-xl border flex flex-row items-center justify-between gap-2", children: [_jsx("code", { children: type }), _jsx("span", { className: "text-xs", children: body.required ? 'Required' : 'Optional' })] }), body.description ? _jsx(Markdown, { text: body.description }) : null, _jsx(Schema, { name: "body", schema: (body.content[type].schema ?? {}), required: body.required, ctx: {
                        readOnly: method.method === 'GET',
                        writeOnly: method.method !== 'GET',
                        render: ctx,
                    } })] }));
    }
    if (method.responses && ctx.showResponseSchema !== false) {
        const statuses = Object.keys(method.responses);
        responseNode = (_jsxs(_Fragment, { children: [heading(headingLevel, 'Response Body', ctx), _jsx(Tabs, { items: statuses, groupId: "fumadocs_openapi_responses", children: statuses.map((status) => (_jsx(ResponseTab, { status: status, operation: method, ctx: ctx }, status))) })] }));
    }
    const parameterNode = Object.entries(ParamTypes).map(([type, title]) => {
        const params = method.parameters?.filter((param) => param.in === type);
        if (!params || params.length === 0)
            return;
        return (_jsxs(Fragment, { children: [heading(headingLevel, title, ctx), _jsx("div", { className: "flex flex-col gap-4", children: params.map((param) => (_jsx(Schema, { name: param.name, schema: {
                            ...param.schema,
                            description: param.description ?? param.schema?.description,
                            deprecated: (param.deprecated ?? false) ||
                                (param.schema?.deprecated ?? false),
                        }, parseObject: false, required: param.required, ctx: {
                            readOnly: method.method === 'GET',
                            writeOnly: method.method !== 'GET',
                            render: ctx,
                        } }, param.name))) })] }, type));
    });
    if (method.callbacks) {
        callbacksNode = (_jsxs(_Fragment, { children: [heading(headingLevel, 'Webhooks', ctx), Object.entries(method.callbacks).map(([name, callback]) => (_jsx(WebhookCallback, { callback: callback, ctx: ctx, headingLevel: headingLevel }, name)))] }));
    }
    const security = (method.security ??
        ctx.schema.document.security ??
        []).filter((v) => Object.keys(v).length > 0);
    const info = (_jsxs(ctx.renderer.APIInfo, { head: headNode, method: method.method, route: path, children: [type === 'operation' ? (ctx.disablePlayground ? (_jsxs("div", { className: "flex flex-row items-center gap-2.5 p-3 rounded-xl border bg-fd-card text-fd-card-foreground not-prose", children: [_jsx(MethodLabel, { className: "text-xs", children: method.method }), _jsx("code", { className: "flex-1 overflow-auto text-nowrap text-[13px] text-fd-muted-foreground", children: path })] })) : (_jsx(ctx.renderer.APIPlayground, { path: path, method: method, ctx: ctx }))) : null, security.length > 0 ? (_jsxs(_Fragment, { children: [heading(headingLevel, 'Authorization', ctx), _jsx(AuthSection, { requirements: security, ctx: ctx })] })) : null, bodyNode, parameterNode, responseNode, callbacksNode] }));
    if (type === 'operation') {
        const examples = getAPIExamples(path, method, ctx);
        return (_jsx(ctx.renderer.API, { children: _jsxs(APIExampleProvider, { route: path, examples: examples, method: method, children: [info, _jsx(APIExample, { examples: examples, method: method, ctx: ctx })] }) }));
    }
    else {
        return info;
    }
}
async function ResponseTab({ status, operation, ctx, }) {
    const response = operation.responses[status];
    const { generateTypeScriptSchema, schema } = ctx;
    const mediaType = response.content
        ? getPreferredType(response.content)
        : null;
    const responseOfType = mediaType ? response.content?.[mediaType] : null;
    const description = responseOfType?.schema?.description ?? response.description ?? '';
    let ts;
    if (generateTypeScriptSchema) {
        ts = await generateTypeScriptSchema(operation, status);
    }
    else if (generateTypeScriptSchema === undefined && responseOfType?.schema) {
        ts = await getTypescriptSchema(responseOfType?.schema, schema.dereferenceMap);
    }
    return (_jsxs(Tab, { value: status, children: [_jsx(Markdown, { text: description }), ts && _jsx(CopyResponseTypeScript, { code: ts }), responseOfType?.schema && (_jsx(Schema, { name: "response", schema: responseOfType.schema, required: true, ctx: {
                    render: ctx,
                    writeOnly: false,
                    readOnly: true,
                } }))] }));
}
function WebhookCallback({ callback, ctx, headingLevel, }) {
    return Object.entries(callback).map(([path, pathItem]) => {
        const pathNodes = methodKeys.map((method) => {
            const operation = pathItem[method];
            if (!operation)
                return null;
            return (_jsx(Operation, { type: "webhook", hasHead: true, path: path, headingLevel: headingLevel + 1, method: createMethod(method, pathItem, operation), ctx: ctx }, method));
        });
        return _jsx(Fragment, { children: pathNodes }, path);
    });
}
function AuthSection({ ctx: { schema: { document }, renderer, }, requirements, }) {
    let id = 0;
    const info = [];
    for (const requirement of requirements) {
        for (const schema of getSecurities(requirement, document)) {
            const prefix = getSecurityPrefix(schema);
            const scopeElement = schema.scopes.length > 0 ? (_jsxs("p", { children: ["Scope: ", _jsx("code", { children: schema.scopes.join(', ') })] })) : null;
            if (schema.type === 'http' || schema.type === 'oauth2') {
                info.push(_jsxs(renderer.Property, { name: "Authorization", type: prefix ? `${prefix} <token>` : '<token>', required: true, children: [schema.description ? _jsx(Markdown, { text: schema.description }) : null, _jsxs("p", { children: ["In: ", _jsx("code", { children: "header" })] }), scopeElement] }, id++));
            }
            if (schema.type === 'apiKey') {
                info.push(_jsxs(renderer.Property, { name: schema.name, type: "<token>", children: [schema.description ? _jsx(Markdown, { text: schema.description }) : null, _jsxs("p", { children: ["In: ", _jsx("code", { children: schema.in }), scopeElement] })] }, id++));
            }
            if (schema.type === 'openIdConnect') {
                info.push(_jsxs(renderer.Property, { name: "OpenID Connect", type: "<token>", required: true, children: [schema.description ? _jsx(Markdown, { text: schema.description }) : null, scopeElement] }, id++));
            }
        }
    }
    return info;
}
